/*****************************************************************************/
/*  File       : appli.c                                                     */
/*                                                                           */
/*  This is the main source c file:                                          */
/*    Firmware code for FX30S LTE-m1 modem to support backup communications  */
/*    for Infraguard-I and Infraguard-II security boards.                    */
/*                                                                           */
/*****************************************************************************/
/*****************************************************************************/
/*  Header files                                                             */
/*****************************************************************************/
#include "legato.h"
#include "interfaces.h"
#include "le_data_interface.h"
#include <pthread.h>
#include <termios.h>
#include <fcntl.h>
#include <string.h>
#include <sys/select.h>
#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netdb.h>

/***************************************************************************/
/*  Defines                                                                */
/***************************************************************************/

#define	BKSP    '\177'			// BackSpace sequence

#define BUFFER_SIZE 512
typedef unsigned char ascii;

//The technology string max length
#define TECH_MAX_LEN        16
#define TECH_MAX_BYTES      (TECH_MAX_LEN+1)

//Current version message
#define Version "\r\n\nAdvantor Systems - FX30S-Infraguard v1.0.0\r\n"


/***************************************************************************/
/*  Local/Global variables                                                 */
/***************************************************************************/

ascii inBuffer[BUFFER_SIZE];
ascii inData[BUFFER_SIZE];
int fd1;
int sock;
int pings = 0;

char	CommandStr[512];			// Command buffer
char	SERVER_IP[15];				// IP Address
char	IPAddress[20];				// IP Address buffer
int		IPPort = 0;					// integer IP Port buffer

//Set boolean flags
bool	DataConnected	= false;	// Flag for Data call running
bool	Registered 		= false;	// Flag for Registered State
bool	SocketOpened	= false;	// Flag for Opened Socket
bool	SocketConnected	= false;	// Flag for Opened Connected
bool	gpio56tmr		= false;	// Flag for GPIO56 timer
bool	gpioPin48		= false;	// Flag for GPIO48 ConnectService

//The Data Connection reference
static le_data_RequestObjRef_t RequestRef = NULL;

//List of technologies to use
static le_data_Technology_t TechList[LE_DATA_MAX] = {};

//List of technology strings
static const char techDico[LE_DATA_MAX][TECH_MAX_LEN] =
{
    "wifi",
    "cellular"
};

// threads
le_thread_Ref_t thread_1, thread_2;

//Reference timer for app session
le_timer_Ref_t gpio56Timer;


/***************************************************************************/
/*  Function prototypes                                                    */
/***************************************************************************/
void GetIPPort ( char *s1);

void ConnectServer (void);

void ResolveDNS (void);

void* ioThread(void* object);

void* ioThread_TCP(void* object);

static void AppRestart(void);

//--------------------------------------------------------------------------------------------------
//
//  This function will set the technologies to use
//
//--------------------------------------------------------------------------------------------------
static void SetTechnologies (void)
{
    int techCounter = 0;
    LE_INFO("Setting the technologies to use for the data connection.");

    // Add 'Cellular' as the first and only technology to use
    LE_ASSERT(LE_OK == le_data_SetTechnologyRank(1, LE_DATA_CELLULAR));
    TechList[techCounter++] = LE_DATA_CELLULAR;
}

//--------------------------------------------------------------------------------------------------
//
//  This function will get the technologies to use
//
//--------------------------------------------------------------------------------------------------
static void CheckTechnologies (void)
{
    int techCounter = 0;

    LE_INFO("Checking the technologies to use for the data connection.");

    // Get first technology to use
    le_data_Technology_t technology = le_data_GetFirstUsedTechnology();

    if (technology != TechList[techCounter])
    {
        LE_ERROR("Unexpected first technology %d, should be %d.", technology, TechList[techCounter]);
        exit(EXIT_FAILURE);
    }


    //else
    //{
    //    techCounter++;
    //}

    // Get next technologies to use
    //while (LE_DATA_MAX != (technology = le_data_GetNextUsedTechnology()))
    //{
    //    if (technology != TechList[techCounter])
    //    {
    //        LE_ERROR("myApp - Unexpected technology %d, should be %d.", technology, TechList[techCounter]);
    //        exit(EXIT_FAILURE);
    //    }
    //    else
    //    {
    //        techCounter++;
    //    }
    //}
}

//--------------------------------------------------------------------------------------------------
//
//  This function will request the data connection
//
//--------------------------------------------------------------------------------------------------
static void ConnectData (void)
{
    if (RequestRef)
    {
        LE_ERROR("A data connection request already exist.");
    }
    else
    {
    	RequestRef = le_data_Request();
    	LE_INFO("Requesting the data connection: %p.", RequestRef);
    }
}

//--------------------------------------------------------------------------------------------------
/**
 *  The opposite of ConnectData, this function will tear down the data connection.
 */
//--------------------------------------------------------------------------------------------------

/*static void DisconnectData (void)
{
    if (!RequestRef)
    {
        LE_ERROR("No data connection reference.");
    }

    // Release the connection
    LE_INFO("Releasing the data connection.");
    le_data_Release(RequestRef);

    // Reset connection reference
    RequestRef = NULL;
}
*/

/***************************************************************************/
/*  Function   : GetIPPort                                                 */
/*-------------------------------------------------------------------------*/
/*  Object     : Check CommandStr and gets the IP address and Port number.  */
/*-------------------------------------------------------------------------*/
/*  Variable Name     |IN |OUT|GLB|  Utilization                           */
/*--------------------+---+---+---+----------------------------------------*/
/*  IPAddress         |   | X |   | Assigns IPAddress for socket comm      */
/*  IPPort            |   | X |   | Assigns IPPort for socket comm         */
/*--------------------+---+---+---+----------------------------------------*/
/***************************************************************************/
void GetIPPort ( char *s1)
{
    int Found  = 0;
    int Spaces = 0;
    int i  = 0;
    int i2 = 0;
    int i3 = 0;
    char s2[10];
    char s3[20];

    while ( s1[i] != '\0' && !Found )
    {
        if ( s1[i] == ' ')
        {
            Spaces++;
        }
        if ( Spaces == 1 && s1[i] != ' ' )
        {
            s3[i3] = s1[i];
            i3++;
        }
        if ( Spaces == 2 )
        {
           Found = 1;
        }
        i++;
    }
    s3[i3] = '\0';
    for ( i2 = 0; i2 <= 10; i2++ )
    {
        s2[i2] = s1[i];
        if ( s1[i] == '\0') break;
        i++;
    }
    strcpy ( IPAddress, s3);
    IPPort = atoi (s2);
}

//-------------------------------------------------------------------------------------------------
//
//  CheckStatus - This function checks current modem information statuses and displays them
//				  to the screen.
//
//-------------------------------------------------------------------------------------------------
void CheckStatus (void)
{
	LE_INFO ("Start CheckStatus...");
	char nameStr[100] = {0};
	le_mrc_ConnectService();
	le_mrc_GetCurrentNetworkName(nameStr, 100);

	//Display version message on serial port at startup:
	write(fd1, Version, strlen(Version));

	//Display Cell Network:
	write(fd1, "\nCurrent Cellular Network:    ", 30);
	write(fd1, nameStr, strlen(nameStr));
	write(fd1, "\r\n", 2);

	//Display registration state:
	le_mrc_NetRegState_t RegState;
	le_mrc_GetNetRegState(&RegState);
	write(fd1, "Current Registration State:  ", 29);

	switch (RegState)
	{
	case LE_MRC_REG_NONE:
		write(fd1, "None\r\n", 6);
		break;

	case LE_MRC_REG_HOME:
		write(fd1, "Home\r\n", 6);
		break;

	case LE_MRC_REG_SEARCHING:
		write(fd1, "Searching...\r\n", 14);
		break;

	case LE_MRC_REG_DENIED:
		write(fd1, "Denied\r\n", 8);
		break;

	case LE_MRC_REG_ROAMING:
		write(fd1, "Roaming\r\n", 9);
		break;

	case LE_MRC_REG_UNKNOWN:
	default:
		write(fd1, "Unknown\r\n", 9);
		break;
	}

	//Display signal strength:
	uint32_t SigQuality;
	le_mrc_GetSignalQual(&SigQuality);
	write(fd1, "Current Signal Strength:     ", 29);

	switch (SigQuality)
	{
	case 5:
		write(fd1, "Very strong signal strength (5)\r\n", 33);
		break;

	case 4:
		write(fd1, "Strong signal strength (4)\r\n", 28);
		break;

	case 3:
		write(fd1, "Good signal strength (3)\r\n", 26);
		break;

	case 2:
		write(fd1, "Weak signal strength (2)\r\n", 26);
		break;

	case 1:
		write(fd1, "Very weak signal strength (1)\r\n", 31);
		break;

	case 0:
	default:
		write(fd1, "No signal (0)\r\n", 16);
		break;
	}

	//Display current RAT:
	le_mrc_Rat_t RatPref;
	le_mrc_GetRadioAccessTechInUse(&RatPref);
	write(fd1, "Current radio technology:    ", 29);

	switch (RatPref)
	{
	case LE_MRC_RAT_GSM:
		write(fd1, "GSM\r\n", 5);
		break;

	case LE_MRC_RAT_UMTS:
		write(fd1, "UMTS\r\n", 6);
		break;

	case LE_MRC_RAT_TDSCDMA:
		write(fd1, "TDSCDMA\r\n", 9);
		break;

	case LE_MRC_RAT_LTE:
		write(fd1, "LTE\r\n", 5);
		break;

	case LE_MRC_RAT_CDMA:
		write(fd1, "CDMA\r\n", 6);
		break;

	default:
		write(fd1, "Not connected.\r\n", 16);
		break;
	}

	write(fd1, "\r\n\n# ", 5);
}


//******************************************************************************
//
// This Thread resolves a problem with the recv() function hanging up with
// received data in the input buffer.
//
// The send() below for some reason triggers the recv() to spit out the buffer
// data which allows for continued processing of communication data.
//
//******************************************************************************
/*
void* Thread_Test(void* object)
{
	char testBuff[1];
	memset (testBuff, 0, sizeof (testBuff));

	while (1)
	{
		if (SocketConnected)
		{
			if( send(sock , testBuff , 1 , 0) < 0)
			{
				LE_INFO("Sent Test failed");
			}
			else
			{
				//LE_INFO("Sent Test Go");
			}
		}
		sleep(1);
	}

} */
//******************************************************************************
//
// This Thread runs the kick for the watchDog timer.
//
//******************************************************************************
void* WatchDog_Thread(void* object)
{
	le_wdog_ConnectService();

	while (1)
	{
		//watchdog kick
		//LE_DEBUG("Sent watchDog kick.");
		le_wdog_Kick();
		sleep(40);
	}

}

/***************************************************************************/
/*  Thread   :        InputThread                                                    */
/*-------------------------------------------------------------------------*/
/*  Object     :                                                           */
/*-------------------------------------------------------------------------*/
/*  Variable Name     |IN |OUT|GLB|  Utilization                           */
/*--------------------+---+---+---+----------------------------------------*/
/*                    |   |   |   |                                        */
/*                    |   |   |   |                                        */
/*--------------------+---+---+---+----------------------------------------*/
/***************************************************************************/
//void* InputThread(void* object)
void* ioThread(void* object)
{

	LE_INFO("ioThread Started.");
	LE_INFO("Waiting for data on serial port.");

	//Display version message on serial port at startup.
	write(fd1, Version, strlen(Version));
	write(fd1, "\r\n\n# ", 5);


	//Start thread_3  TEST
	//le_thread_Ref_t thread_3 = le_thread_Create( "Test_Thread", Thread_Test, NULL );
	//le_thread_Start(thread_3);


	while (1)
	{
		//LE_INFO("Starting while statement.");

		//Read in buffer
    	memset(inBuffer,0,BUFFER_SIZE);
    	memset(inData,0,BUFFER_SIZE);

    	int n=read(fd1, inData, BUFFER_SIZE);
    	if (n < 0) LE_DEBUG("Read uart1 inData failed.");

        int i;

        //Send data to socket if opened.
        if (SocketConnected)
        {
        	LE_DEBUG("Socket is connected - send inData: %s", inData);

        	//If telnet or AT# command is found in the input buffer, need to close
        	if  ((!strncmp((char *)inData, "telnet ", 7 )) || (!strncmp((char *)inData, "AT#", 3 )))
        	{
        		LE_DEBUG("Telnet or AT# command found in input buffer during data mode.");
    			shutdown(sock,SHUT_RD);
    			continue;
        	}

        	//Ping - Ack 	If received ack message, then reset ping_ack counter.
        	if  (!strncmp((char *)inData, "<LEG>", 5 ))
        	{
        		LE_INFO("Received ACK message.");
        		pings = 0;
        	}

            if( send(sock , inData , n , 0) < 0) LE_INFO("Socket data send failed");

            //LE_DEBUG("First HEX sent inData: %x", inData[0]);

        }
        else
        {
            //Echo incoming uart1 data
            write(fd1, inData, n);

            //Process the input buffer received on the serial port.
            for ( i = 0 ; i < n ; i++ )
        	{
        		/* test for carriage return */
        		if ((strlen(CommandStr) == 0) && (inData[i] == '\r'))
        		{
        			LE_INFO ("Received carriage return" );
        			write(fd1, "\r\n# ", 4);
        			continue;
        		}

        		/* AT mode : echo each char until '\r' */
        		if (strlen(CommandStr) >= 0)
        		{
        			//Test for Backspace
        			if (inData[i] == BKSP)
        			{
        				/* Backspace */
        				if (strlen(CommandStr) > 0)
        				{
        					LE_INFO ("Performing Backspace");

        					/* Erase last character */
        					CommandStr [strlen(CommandStr) - 1] = 0;
            		        LE_INFO("CommandStr after Backspace: %s", CommandStr);
        				}
        				continue;
        			}

        			/* Check for not carriage return */
        			if (inData[i] != '\r')
        			{
        				/* Add input char to command field */
        				CommandStr [strlen(CommandStr) + 1] = 0;
        				CommandStr [strlen(CommandStr)] = inData[i];
        			}

        			/* Check for carriage return */
        			if (inData[i] == '\r')
        			{
        				// Check for Telnet command
        				if  (!strncmp(CommandStr, "telnet ", 7 ) )
        				{
        					LE_INFO ("Telnet command sent to GetIPPort function: %s ", CommandStr);
        					GetIPPort (CommandStr);
        					LE_INFO ("GetIPPort finished: %s:%i", IPAddress, IPPort );

        					if ((Registered) && (DataConnected)) //If modem registered to cell network proceed...
        					{
        						if (!SocketConnected) //If no socket opened then establish session.
        						{
        						    // Start the default data connection
        						    ConnectServer();
        						}
        						else { write (fd1, "\n\rSession in progress - Please wait\n\r# ", 39); }
        					}
        					else
        					{
            					LE_INFO ("Not registered or no data connection." );
        						write (fd1, "\n\rError - Not Registered\n\r# ",28 );
        					}
        				}
        				// Check for Status command
        				else if  (!strncmp(CommandStr, "status", 6 ) )
        				{
        					LE_INFO ("Status command received.");
        					CheckStatus();
        				}
        				// Check for Restart command
        				else if  (!strncmp(CommandStr, "restart", 7 ) )
        				{
        					LE_INFO ("Restart command received.");
        					AppRestart();
        				}
        				// Send Command
        				else
        				{
        					LE_INFO ("Received carriage return" );
                			write(fd1, "\r\n# ", 4);
        				}
        				// Reset Command
        				CommandStr [0] = 0;
        			}
        		}
        	}
        }
        //Reset buffer inData
        inData[0]=0;
	}
}

//--------------------------------------------------------------------------------------------------
//
//  OpenPort - this function opens and set controls for the uart1 on the FX30s.
//
//--------------------------------------------------------------------------------------------------
int OpenPort(const char* port)
{
	LE_INFO("Opening serial %s", port);
     int fd=open(port, O_RDWR | O_NOCTTY | O_NDELAY);
     if(fd==-1)
     {
    	 LE_INFO("Error opening serial port %s... exiting", port);
          exit(-1);
     }

     fcntl(fd, F_SETFL, 0); /* Reads will be blocking */
     struct termios options;
     tcgetattr(fd, &options);
     //(void)cfsetispeed(&options, B115200); /* (void) is to stop warning in cygwin */
     //(void)cfsetospeed(&options, B115200);
     (void)cfsetispeed(&options, B38400); /* (void) is to stop warning in cygwin */
     (void)cfsetospeed(&options, B38400);
     options.c_cflag &= ~CSIZE;
     options.c_cflag |= CS8;  /* 8 bits */
     options.c_cflag &= ~CSTOPB; /* 1 stop bit */
     options.c_cflag &= ~PARENB; /* no parity */
     options.c_cflag &= ~PARODD;
     //options.c_cflag &= ~CRTSCTS; /* HW flow control off */
     options.c_cflag |= CRTSCTS; /* HW flow control on */
     options.c_lflag =0; /* RAW input */
     options.c_iflag = 0;            /* SW flow control off, no parity checks etc */
     //options.c_iflag |= IXON | IXOFF | IXANY;
     options.c_oflag &= ~OPOST; /* RAW output */
     //options.c_cc[VTIME]=10; /* 1 sec */
     options.c_cc[VTIME]=1; /* .5 sec */
     //options.c_cc[VTIME]=0; /* 1 sec */
     //options.c_cc[VMIN]=BUFFER_SIZE;
     //options.c_cc[VMIN]=1;
     options.c_cc[VMIN]=200;
     options.c_cflag |= (CLOCAL | CREAD);
     tcsetattr(fd, TCSAFLUSH, &options);

     //int status;
     //ioctl(fd, TIOCMGET, &status);
     //status |= TIOCM_DTR;
     //status |= TIOCM_CTS;
     //status |= TIOCM_LE;
     //status |= TIOCM_DSR;
     //status |= TIOCM_CAR;
     //status &= ~TIOCM_DTR;
     //status &= ~TIOCM_CTS;
     //status &= ~TIOCM_LE;
     //status &= ~TIOCM_DSR;
     //status &= ~TIOCM_CAR;
     //ioctl(fd, TIOCMSET, status);

     /*
     if (status & TIOCM_DTR)     { LE_INFO("TIOCM_DTR is not set");   }
     else     { LE_INFO("TIOCM_DTR is set"); }

     if (status & TIOCM_DSR)     { LE_INFO("TIOCM_DSR is not set");   }
     else     { LE_INFO("TIOCM_DSR is set"); }

     if (status & TIOCM_CTS)  { LE_INFO("TIOCM_CTS is not set");  }
     else { LE_INFO("TIOCM_CTS is set"); }
     */

     //TIOCMBIS;

     //sleep(60);

     return fd;
}

//--------------------------------------------------------------------------------------------------
//
//  ResolveDNS function resolves the supplied DNS and returns the IP address.
//
//--------------------------------------------------------------------------------------------------
void ResolveDNS (void)
{
	struct addrinfo hints = {0};
    struct addrinfo *addrs = NULL;
    char addrstr[100];
    void *ptr;

    memset(&hints, 0, sizeof hints);

    hints.ai_family     = AF_INET;
    hints.ai_socktype   = SOCK_STREAM;
    hints.ai_protocol   = IPPROTO_TCP;

    if (getaddrinfo(IPAddress, NULL, &hints, &addrs) != 0)
    {
    	LE_INFO("ERROR, no such host: ");
    	LE_ERROR("getaddrinfo failed with errno %d (%m)", errno);
    }

    while (addrs)
    {
    	inet_ntop (addrs->ai_family,addrs->ai_addr->sa_data,addrstr, 100);

    	switch (addrs->ai_family)
    	{
    	case AF_INET:
    		ptr = &((struct sockaddr_in *) addrs->ai_addr)->sin_addr;
    		break;
    	case AF_INET6:
    		ptr = &((struct sockaddr_in6 *) addrs->ai_addr)->sin6_addr;
    		break;
    	}

    	inet_ntop (addrs->ai_family, ptr, addrstr, 100);
    	LE_INFO("Socket IP address is: %s", addrstr);
    	addrs = addrs->ai_next;
    }

    //Returns the DSN IP address
    strcpy ( SERVER_IP, addrstr);
}

//--------------------------------------------------------------------------------------------------
//
//  AppRestart function restart app to recover from problems.
//
//--------------------------------------------------------------------------------------------------
static void AppRestart(void)
{
	//system("/sbin/reboot -f");
	//system("/mnt/legato/system/bin/app restart FX30S_Infraguard");
	//system("/mnt/legato/system/bin/app restartLegato");
}

//--------------------------------------------------------------------------------------------------
//
//  ConnectServer function connects to the socket and starts the socket thread.
//
//--------------------------------------------------------------------------------------------------
void ConnectServer (void)
{
	LE_INFO("ConnectServer started...");

	struct sockaddr_in server;

	// Resolve the DNS host name to IP address.
	ResolveDNS ();

    //Create socket
	sock = socket(AF_INET , SOCK_STREAM , 0);
	if (sock == -1)
	{
		LE_INFO("Could not create socket.");
	}
	else
	{
		LE_INFO("Socket created successfully.");
	}

	//Assign values to sockaddr_in
    server.sin_addr.s_addr = inet_addr(SERVER_IP);
    server.sin_family = AF_INET;
    server.sin_port = htons(IPPort);

    //Connect to cellhost server
    if (connect(sock, (struct sockaddr *)&server, sizeof(server)) < 0)
    {
    	LE_INFO("Socket connection to server failed. Error");
    	LE_ERROR("Socket Connect failed with errno %d (%m)", errno);
    	if (errno == 101)
    	{
    		//le_framework_Restart();
    		AppRestart();
    	}
    	write(fd1, "\r\nSocket connection to server failed. Error\r\n# ", 47);
    	shutdown(sock,SHUT_RDWR);
    	LE_DEBUG("Socket closed");
    }
    else
    {
    	LE_INFO("Socket Connected to %s:%d", SERVER_IP, IPPort);
    	SocketConnected = true;
    	char *ConnectMSG = ("\n\rConnected to ");
    	write(fd1, ConnectMSG, strlen(ConnectMSG));
    	write(fd1, SERVER_IP, sizeof(SERVER_IP));
    	write(fd1, "\r\n", 2);
    	sleep(1);

    	//Start thread_2
    	le_thread_Ref_t thread_2 = le_thread_Create( "io_TCP_Thread", ioThread_TCP, NULL );
    	le_thread_Start(thread_2);
    }
}

//--------------------------------------------------------------------------------------------------
//
//  Thread - ioThread, handles communication and control of socket connection.
//
//--------------------------------------------------------------------------------------------------
void* ioThread_TCP(void* object)
{

	//Connect service to gpioPin48
	le_gpioPin48_ConnectService();

	//Turn on LED green
	le_gpioPin48_Deactivate();

	ascii server_reply[512];

	LE_INFO("Waiting for data on TCP port.");

	while(1)
	{
		memset (server_reply, 0, sizeof (server_reply));

		//Receive data from the server (cellhost)
		if (recv(sock , server_reply , 1 , MSG_PEEK | MSG_DONTWAIT) == 0)
		{
			//Received FIN from server - close connection and socket.
			LE_INFO("recv received FIN, shutting down socket");
			shutdown(sock,SHUT_RDWR);
			SocketConnected = false;
			pings = 0;
			le_gpioPin48_Activate();	//Turn off LED
			le_gpioPin48_DisconnectService();
			write(fd1, "\r\nConnection to host lost.\r\n# ", 30);
			//Add memrelease to thread here for server_reply
			le_thread_Exit(thread_2);
		}
		else
		{
			//Received data sent to uart1.
			int Length = (recv(sock , server_reply , BUFFER_SIZE , 0));
			LE_DEBUG("Server reply: %s, sending to uart1", server_reply);
			write(fd1, server_reply, Length);

        	//Ping - Ack 	If receive 3 consecutive ping messages from cellhost without an ack from cellcomm,
			//then close the connection.
        	if  (!strncmp((char *)server_reply, "<LEG>", 5 ))
        	{
        		LE_INFO("Received PING message.");
        		pings++;
        		if (pings > 2)
        		{
            		LE_INFO("Received 3 consecutive PINGs with no ACKs, closing socket connection.");
        			shutdown(sock,SHUT_RDWR);
        		}
        	}

		}
	}
}

//--------------------------------------------------------------------------------------------------
//
//  Event callback for data connection state changes.
//
//--------------------------------------------------------------------------------------------------
static void DcsStateHandler (const char* intfName, bool isConnected, void* contextPtr)
{
    if (isConnected)
    {
        // Get the technology used for the data connection
        le_data_Technology_t usedTech = le_data_GetTechnology();
        if (LE_DATA_MAX == usedTech)
        {
            LE_ERROR("Unknown technology used for the data connection of '%s'!", intfName);
        }
        else
        {
            LE_INFO("'%s' connected using the technology '%s'!", intfName, techDico[usedTech]);
        }

        DataConnected = true;
    }
    else
    {
        LE_INFO("'%s' disconnected!", intfName);
        DataConnected = false;
        //DisconnectData();
        ConnectData();
    }

}

// -------------------------------------------------------------------------------------------------
//
//  Get a string for a given network state enumeration.
//  @return
//       A string that best represents the given enumerated network state.
//  @note
//       Do *not* free the returned string.
//
// -------------------------------------------------------------------------------------------------
static const char* GetNetStateString (le_mrc_NetRegState_t state)
{
    char* string = NULL;

    switch (state)
    {
        case LE_MRC_REG_NONE:
            string = "off of the network";
            break;

        case LE_MRC_REG_HOME:
            string = "registered on its home network";
            break;

        case LE_MRC_REG_SEARCHING:
            string = "searching for network";
            break;

        case LE_MRC_REG_DENIED:
            string = "denied access to the network";
            break;

        case LE_MRC_REG_ROAMING:
            string = "registered on a roaming network";
            break;

        case LE_MRC_REG_UNKNOWN:
        default:
            string = "in an unknown state";
            break;
    }
    return string;
}

// -------------------------------------------------------------------------------------------------
//
//  An event callback that is run every time the modem's state changes.  When the modem becomes
//  attached to a network, the flag "Registered" is set to true, and flase when registration lost.
//
// -------------------------------------------------------------------------------------------------
static void NetRegStateHandler (le_mrc_NetRegState_t state, void* contextPtr)
{
	// For traceability, make sure that this event is recorded.
    LE_INFO("%s", GetNetStateString(state));

    // If we are going back on net, and have been configured to do so, send our "on network" message
    // now.
    if ((state == LE_MRC_REG_HOME) || (state == LE_MRC_REG_ROAMING))
    {
    	LE_INFO("Registration - Home or Roaming.");
    	Registered = true;
        le_timer_Restart(gpio56Timer);					//Restart timer
    }
    else if ((state == LE_MRC_REG_UNKNOWN) || (state == LE_MRC_REG_NONE))
	{
    	LE_INFO("Registration - Unknown or Off Network.");
    	Registered = false;
        le_timer_Stop(gpio56Timer);							//stop timer
	}
    else if ((state == LE_MRC_REG_SEARCHING))
	{
    	LE_INFO("Registration - Searching for network.");
	}
}

//--------------------------------------------------------------------------------------------------
//
// Configure APN based on cell provider
//
//--------------------------------------------------------------------------------------------------
static void SetConfiguration()
{
	static const char TMobile[] = "T-Mobile";
	static const char ATT[] = "AT&T";
	static const char Verizon[] = "Verizon";

	//#define PROFILE 1
 	le_mdc_ProfileRef_t PROFILE;

    char   apnTMobile[LE_MDC_APN_NAME_MAX_LEN] = 	"fast.t-mobile.com";	// access point network
    char   apnVerizon[LE_MDC_APN_NAME_MAX_LEN] = 	"vzwinternet";			// access point network
    char   apnATT[LE_MDC_APN_NAME_MAX_LEN] = 		"broadband";			// access point network

    le_result_t res;
    char nameStr[100] = {0};

    while (res)
    {
        LE_DEBUG("Get Network Name");
        res = le_mrc_GetCurrentNetworkName(nameStr, 100);
        if (res) sleep(4);
    }

    LE_ASSERT(res == LE_OK);
    LE_INFO("Cellular Network Name: %s",nameStr);

    PROFILE = le_mdc_GetProfile(1);

    if ( strncmp(nameStr, TMobile, sizeof(TMobile)) == 0 )
    {
        LE_INFO("Set APN to: fast.t-mobile.com");
    	LE_ASSERT(le_mdc_SetAPN(PROFILE, apnTMobile) == LE_OK );
    }
    else if ( strncmp(nameStr, Verizon, sizeof(Verizon)) == 0 )
    {
        LE_INFO("Set APN to: vzwinternet");
    	LE_ASSERT(le_mdc_SetAPN(PROFILE, apnVerizon) == LE_OK );
    }
    else if ( strncmp(nameStr, ATT, sizeof(ATT)) == 0 )
    {
        LE_INFO("Set APN to: broadband");
    	LE_ASSERT(le_mdc_SetAPN(PROFILE, apnATT) == LE_OK );
    }

}

//-------------------------------------------------------------------------------------------------
//
//  gpio56TimerExpiredHandler - This function creates a pulse on output Pin03 (gpioPin56) which
//								the Infraguard panel uses to supervise the registration state.
//
//-------------------------------------------------------------------------------------------------
static void gpio56TimerExpiredHandler(le_timer_Ref_t  timerRef)
{
//	LE_DEBUG("gpio56Timer - 3 sec passed");

	if (gpio56tmr)
    {
        le_gpioPin56_Activate();
        gpio56tmr = false;
    }
    else
    {
        le_gpioPin56_Deactivate();
        gpio56tmr = true;
    }

}

//-------------------------------------------------------------------------------------------------
//
// Main - COMPONENT_INIT
//
//-------------------------------------------------------------------------------------------------
COMPONENT_INIT
{
	LE_INFO("FX30S_Infraguard APP - START");

	//Register handler for data connection state change
    le_data_AddConnectionStateHandler(DcsStateHandler, NULL);

    //Disable Polling Timer
    le_avc_SetPollingTimer(0);

    //Testing *********************
    SetConfiguration();

    //Register network state handler
    le_mrc_AddNetRegStateEventHandler(NetRegStateHandler, NULL);

    //Check initial Network Register State and set flag
    le_mrc_NetRegState_t RegState;
    le_mrc_GetNetRegState(&RegState);
    if ((RegState == LE_MRC_REG_HOME) || (RegState == LE_MRC_REG_ROAMING))
    {
    	LE_INFO("Registration Home or Roaming.");
    	Registered = true;
    }

    //Set technologies to use
    SetTechnologies();

    //Check if the technologies list was correctly updated
    CheckTechnologies();

    //Set timer for registration supervision - control gpioPin56
    LE_INFO("Starting gpioPin56 timer");
    gpio56Timer = le_timer_Create("GPIO56Timer");	//Create timer
    le_clk_Time_t avcInterval = { 3, 0};			//run for 20 seconds
    le_timer_SetInterval(gpio56Timer, avcInterval);
    le_timer_SetRepeat(gpio56Timer, 0);				//set repeat to always
    le_timer_SetHandler(gpio56Timer, gpio56TimerExpiredHandler);	//callback function
    le_timer_Start(gpio56Timer);					//start timer

    //Open serial port
    LE_INFO("Opening serial port");
	fd1=OpenPort("/dev/ttyHSL0");

	//Release gpioPin48 from main thread
	le_gpioPin48_Activate();	//Turn off green LED if on
	le_gpioPin48_DisconnectService();

	//Start thread_1 - Read_Uart_Thread
	thread_1 = le_thread_Create( "Read_Uart_Thread", ioThread, NULL );
	le_thread_Start(thread_1);

	//Start thread_4  WatchDog for Timer-Kick
	le_thread_Ref_t thread_4 = le_thread_Create( "WatchDog_Thread", WatchDog_Thread, NULL );
	le_thread_Start(thread_4);

	//Open data connect service - cm data connect
	ConnectData();

}

